@@ -40,6 +40,7 @@
 
 G_DEFINE_QUARK (e-soup-session-error-quark, e_soup_session_error)
 
+#define E_SOUP_SESSION_MESSAGE_BYTES_KEY "e-soup-session-message-bytes"
 #define BUFFER_SIZE 16384
 
 struct _ESoupSessionPrivate {
@@ -145,12 +146,16 @@ e_soup_session_ensure_auth_usage (ESoupSession *session,
 		}
 
 		if (!g_uri) {
-			ESourceWebdav *extension;
 			ESource *source;
 
 			source = e_soup_session_get_source (session);
-			extension = e_source_get_extension (source, E_SOURCE_EXTENSION_WEBDAV_BACKEND);
-			g_uri = e_source_webdav_dup_uri (extension);
+
+			if (source) {
+				ESourceWebdav *extension;
+
+				extension = e_source_get_extension (source, E_SOURCE_EXTENSION_WEBDAV_BACKEND);
+				g_uri = e_source_webdav_dup_uri (extension);
+			}
 		}
 	}
 
@@ -160,7 +165,7 @@ e_soup_session_ensure_auth_usage (ESoupSession *session,
 	   but do not set the same 'soup_auth' when it's already set (which can happen, when the session
 	   is reused by multiple sources, which connect to the same server, with the same user.
 	   See https://gitlab.gnome.org/GNOME/libsoup/-/issues/196 for more information. */
-	if (!e_soup_session_auth_already_set_locked (session, g_uri, soup_auth, &auth_was_set)) {
+	if (g_uri && !e_soup_session_auth_already_set_locked (session, g_uri, soup_auth, &auth_was_set)) {
 		if (auth_was_set)
 			soup_auth_manager_clear_cached_credentials (auth_manager);
 
@@ -169,7 +174,7 @@ e_soup_session_ensure_auth_usage (ESoupSession *session,
 
 	g_rec_mutex_unlock (&session->priv->session_lock);
 
-	if (!in_g_uri)
+	if (!in_g_uri && g_uri)
 		g_uri_unref (g_uri);
 }
 
@@ -191,6 +196,12 @@ e_soup_session_setup_bearer_auth (ESoupSession *session,
 
 	source = e_soup_session_get_source (session);
 
+	if (!source) {
+		/* Do not localize this error message, it should not get into the UI */
+		g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED, "No ESource specified");
+		return FALSE;
+	}
+
 	success = e_source_get_oauth2_access_token_sync (source, cancellable,
 		&access_token, &expires_in_seconds, error);
 
@@ -309,7 +320,7 @@ e_soup_session_maybe_prepare_auth (ESoupSession *session,
 
 	source = e_soup_session_get_source (session);
 
-	if (e_source_has_extension (source, E_SOURCE_EXTENSION_AUTHENTICATION)) {
+	if (source && e_source_has_extension (source, E_SOURCE_EXTENSION_AUTHENTICATION)) {
 		ESourceAuthentication *extension;
 
 		extension = e_source_get_extension (source, E_SOURCE_EXTENSION_AUTHENTICATION);
@@ -405,6 +416,11 @@ e_soup_session_authenticate_cb (SoupMessage *message,
 		g_warn_if_fail ((gpointer) session->priv->using_bearer_auth == (gpointer) auth);
 		g_clear_object (&session->priv->using_bearer_auth);
 		session->priv->using_bearer_auth = E_SOUP_AUTH_BEARER (auth);
+	} else if (session->priv->using_bearer_auth) {
+		/* This can mean the bearer auth expired, then a Basic auth is used by the libsoup;
+		   that's not meant to be done here, thus fail early. */
+		g_mutex_unlock (&session->priv->property_lock);
+		return FALSE;
 	}
 
 	if (retrying && !session->priv->auth_prefilled) {
@@ -439,7 +455,7 @@ e_soup_session_authenticate_cb (SoupMessage *message,
 	credentials = e_soup_session_dup_credentials (session);
 
 	username = credentials ? e_named_parameters_get (credentials, E_SOURCE_CREDENTIAL_USERNAME) : NULL;
-	if ((!username || !*username) &&
+	if ((!username || !*username) && session->priv->source &&
 	    e_source_has_extension (session->priv->source, E_SOURCE_EXTENSION_AUTHENTICATION)) {
 		ESourceAuthentication *auth_extension;
 
@@ -467,10 +483,11 @@ e_soup_session_set_source (ESoupSession *session,
 			   ESource *source)
 {
 	g_return_if_fail (E_IS_SOUP_SESSION (session));
-	g_return_if_fail (E_IS_SOURCE (source));
+	if (source)
+		g_return_if_fail (E_IS_SOURCE (source));
 	g_return_if_fail (!session->priv->source);
 
-	session->priv->source = g_object_ref (source);
+	session->priv->source = source ? g_object_ref (source) : NULL;
 }
 
 static void
@@ -710,7 +727,9 @@ e_soup_session_get_log_level (ESoupSession *session)
  * e_soup_session_get_source:
  * @session: an #ESoupSession
  *
- * Returns: (transfer none): Associated #ESource with the @session.
+ * Returns an #ESource associated with the @session, if such was set in the creation time.
+ *
+ * Returns: (transfer none) (nullable): Associated #ESource with the @session, or %NULL.
  *
  * Since: 3.26
  **/
@@ -815,10 +834,10 @@ e_soup_session_get_authentication_requires_credentials (ESoupSession *session)
 /**
  * e_soup_session_get_ssl_error_details:
  * @session: an #ESoupSession
- * @out_certificate_pem: (out): return location for a server TLS/SSL certificate
- *   in PEM format, when the last operation failed with a TLS/SSL error
- * @out_certificate_errors: (out): return location for a #GTlsCertificateFlags,
- *   with certificate error flags when the operation failed with a TLS/SSL error
+ * @out_certificate_pem: (out) (optional): return location for a server TLS/SSL certificate
+ *   in PEM format, when the last operation failed with a TLS/SSL error, or %NULL
+ * @out_certificate_errors: (out) (optional): return location for a #GTlsCertificateFlags,
+ *   with certificate error flags when the operation failed with a TLS/SSL error, or %NULL
  *
  * Populates @out_certificate_pem and @out_certificate_errors with the last values
  * returned on #G_TLS_ERROR_BAD_CERTIFICATE error.
@@ -833,8 +852,6 @@ e_soup_session_get_ssl_error_details (ESoupSession *session,
 				      GTlsCertificateFlags *out_certificate_errors)
 {
 	g_return_val_if_fail (E_IS_SOUP_SESSION (session), FALSE);
-	g_return_val_if_fail (out_certificate_pem != NULL, FALSE);
-	g_return_val_if_fail (out_certificate_errors != NULL, FALSE);
 
 	g_mutex_lock (&session->priv->property_lock);
 	if (!session->priv->ssl_info_set) {
@@ -842,14 +859,91 @@ e_soup_session_get_ssl_error_details (ESoupSession *session,
 		return FALSE;
 	}
 
-	*out_certificate_pem = g_strdup (session->priv->ssl_certificate_pem);
-	*out_certificate_errors = session->priv->ssl_certificate_errors;
+	if (out_certificate_pem)
+		*out_certificate_pem = g_strdup (session->priv->ssl_certificate_pem);
+
+	if (out_certificate_errors)
+		*out_certificate_errors = session->priv->ssl_certificate_errors;
 
 	g_mutex_unlock (&session->priv->property_lock);
 
 	return TRUE;
 }
 
+/**
+ * e_soup_session_handle_authentication_failure:
+ * @session: an #ESoupSession
+ * @credentials: (nullable): credentials used for the authentication
+ * @op_error: a #GError of the authentication operation
+ * @out_auth_result: (out): an #ESourceAuthenticationResult with an authentication result
+ * @out_certificate_pem: (out) (optional): return location for a server TLS/SSL certificate
+ *   in PEM format, when the last operation failed with a TLS/SSL error, or %NULL
+ * @out_certificate_errors: (out) (optional): return location for a #GTlsCertificateFlags,
+ *   with certificate error flags when the operation failed with a TLS/SSL error, or %NULL
+ * @error: return location for a #GError, or %NULL
+ *
+ * Handles authentication failure and sets appropriate value to the @out_auth_result
+ * for the provided @op_error and used @credentials. Converts the @op_error
+ * into an appropriate error returned in the @error.
+ *
+ * Also updates connection status on the associated #ESource with the @session.
+ *
+ * Since: 3.46
+ **/
+void
+e_soup_session_handle_authentication_failure (ESoupSession *session,
+					      const ENamedParameters *credentials,
+					      const GError *op_error,
+					      ESourceAuthenticationResult *out_auth_result,
+					      gchar **out_certificate_pem,
+					      GTlsCertificateFlags *out_certificate_errors,
+					      GError **error)
+{
+	ESource *source;
+	gboolean requires_credentials;
+	gboolean credentials_empty;
+	gboolean is_tls_error;
+
+	g_return_if_fail (E_IS_SOUP_SESSION (session));
+	g_return_if_fail (out_auth_result != NULL);
+
+	source = e_soup_session_get_source (session);
+	requires_credentials = e_soup_session_get_authentication_requires_credentials (session);
+	credentials_empty = (!credentials || !e_named_parameters_count (credentials) ||
+		(e_named_parameters_count (credentials) == 1 && e_named_parameters_exists (credentials, E_SOURCE_CREDENTIAL_SSL_TRUST))) &&
+		requires_credentials;
+	is_tls_error = g_error_matches (op_error, G_TLS_ERROR, G_TLS_ERROR_BAD_CERTIFICATE);
+
+	*out_auth_result = E_SOURCE_AUTHENTICATION_ERROR;
+
+	if (g_error_matches (op_error, E_SOUP_SESSION_ERROR, SOUP_STATUS_FORBIDDEN) && credentials_empty) {
+		*out_auth_result = E_SOURCE_AUTHENTICATION_REQUIRED;
+	} else if (g_error_matches (op_error, E_SOUP_SESSION_ERROR, SOUP_STATUS_UNAUTHORIZED)) {
+		if (credentials_empty)
+			*out_auth_result = E_SOURCE_AUTHENTICATION_REQUIRED;
+		else
+			*out_auth_result = E_SOURCE_AUTHENTICATION_REJECTED;
+	} else if (g_error_matches (op_error, G_IO_ERROR, G_IO_ERROR_CONNECTION_REFUSED) ||
+		   (!requires_credentials && g_error_matches (op_error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))) {
+		*out_auth_result = E_SOURCE_AUTHENTICATION_REJECTED;
+	} else if (!op_error) {
+		g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED, _("Unknown error"));
+	}
+
+	if (op_error)
+		g_propagate_error (error, g_error_copy (op_error));
+
+	if (is_tls_error) {
+		*out_auth_result = E_SOURCE_AUTHENTICATION_ERROR_SSL_FAILED;
+
+		if (source)
+			e_source_set_connection_status (source, E_SOURCE_CONNECTION_STATUS_SSL_FAILED);
+		e_soup_session_get_ssl_error_details (session, out_certificate_pem, out_certificate_errors);
+	} else if (source) {
+		e_source_set_connection_status (source, E_SOURCE_CONNECTION_STATUS_DISCONNECTED);
+	}
+}
+
 static void
 e_soup_session_preset_message (SoupMessage *message)
 {
@@ -1240,6 +1334,56 @@ e_soup_session_prepare_message_send_sync (ESoupSession *session,
 	return NULL;
 }
 
+static GByteArray *
+e_soup_session_read_bytes (SoupMessage *message,
+			   GInputStream *input_stream,
+			   GCancellable *cancellable,
+			   GError **error)
+{
+	GByteArray *bytes;
+	goffset expected_length;
+	gpointer buffer;
+	gsize nread = 0;
+	gboolean success = FALSE;
+
+	expected_length = soup_message_headers_get_content_length (soup_message_get_response_headers (message));
+	if (expected_length > 0)
+		bytes = g_byte_array_sized_new (expected_length > 1024 * 1024 * 10 ? 1024 * 1024 * 10 : expected_length);
+	else
+		bytes = g_byte_array_new ();
+
+	buffer = g_malloc (BUFFER_SIZE);
+
+	while (success = g_input_stream_read_all (input_stream, buffer, BUFFER_SIZE, &nread, cancellable, error),
+	       success && nread > 0) {
+		g_byte_array_append (bytes, buffer, nread);
+	}
+
+	g_free (buffer);
+
+	if (!success)
+		g_clear_pointer (&bytes, g_byte_array_unref);
+
+	return bytes;
+}
+
+static void
+e_soup_session_store_data_on_message (SoupMessage *message,
+				      GInputStream *input_stream,
+				      GCancellable *cancellable)
+{
+	if (input_stream) {
+		GByteArray *bytes;
+
+		bytes = e_soup_session_read_bytes (message, input_stream, cancellable, NULL);
+
+		if (bytes) {
+			g_object_set_data_full (G_OBJECT (message), E_SOUP_SESSION_MESSAGE_BYTES_KEY,
+				bytes, (GDestroyNotify) g_byte_array_unref);
+		}
+	}
+}
+
 static void
 e_soup_session_send_message_ready_cb (GObject *source_object,
 				      GAsyncResult *result,
@@ -1263,11 +1407,18 @@ e_soup_session_send_message_ready_cb (GObject *source_object,
 	message = soup_session_get_async_result_message (session, result);
 
 	if (message) {
+		if (!SOUP_STATUS_IS_SUCCESSFUL (soup_message_get_status (message))) {
+			e_soup_session_store_data_on_message (message, input_stream, NULL);
+			g_clear_object (&input_stream);
+		}
+
 		if (g_error_matches (local_error, G_TLS_ERROR, G_TLS_ERROR_BAD_CERTIFICATE)) {
 			e_soup_session_extract_ssl_data (E_SOUP_SESSION (session), message, &asd->certificate_pem, &asd->certificate_errors);
 		} else if (!local_error && !SOUP_STATUS_IS_SUCCESSFUL (soup_message_get_status (message))) {
+			GByteArray *bytes = e_soup_session_util_get_message_bytes (message);
+
 			if (soup_message_get_status (message) != SOUP_STATUS_FORBIDDEN ||
-			    !e_soup_session_extract_google_daily_limit_error (NULL, 0, &local_error))
+			    !e_soup_session_extract_google_daily_limit_error (bytes ? bytes->data : NULL, bytes ? bytes->len : 0, &local_error))
 				g_set_error_literal (&local_error, E_SOUP_SESSION_ERROR, soup_message_get_status (message),
 					soup_message_get_reason_phrase (message));
 		}
@@ -1502,11 +1653,18 @@ e_soup_session_send_message_sync (ESoupSession *session,
 	if (restarted_id)
 		g_signal_handler_disconnect (message, restarted_id);
 
+	if (!SOUP_STATUS_IS_SUCCESSFUL (soup_message_get_status (message))) {
+		e_soup_session_store_data_on_message (message, input_stream, cancellable);
+		g_clear_object (&input_stream);
+	}
+
 	if (g_error_matches (local_error, G_TLS_ERROR, G_TLS_ERROR_BAD_CERTIFICATE)) {
 		e_soup_session_extract_ssl_data_internal (session, message);
 	} else if (!local_error && !SOUP_STATUS_IS_SUCCESSFUL (soup_message_get_status (message))) {
+		GByteArray *bytes = e_soup_session_util_get_message_bytes (message);
+
 		if (soup_message_get_status (message) != SOUP_STATUS_FORBIDDEN ||
-		    !e_soup_session_extract_google_daily_limit_error (NULL, 0, error))
+		    !e_soup_session_extract_google_daily_limit_error (bytes ? bytes->data : NULL, bytes ? bytes->len : 0, error))
 			g_set_error_literal (&local_error, E_SOUP_SESSION_ERROR, soup_message_get_status (message),
 				soup_message_get_reason_phrase (message));
 	}
@@ -1544,41 +1702,39 @@ e_soup_session_send_message_simple_sync (ESoupSession *session,
 {
 	GInputStream *input_stream;
 	GByteArray *bytes;
-	gint expected_length;
-	gpointer buffer;
-	gsize nread = 0;
 	gboolean success = FALSE;
 
 	g_return_val_if_fail (E_IS_SOUP_SESSION (session), NULL);
 	g_return_val_if_fail (SOUP_IS_MESSAGE (message), NULL);
 
 	input_stream = e_soup_session_send_message_sync (session, message, cancellable, error);
-	if (!input_stream)
+
+	if (!input_stream) {
+		bytes = e_soup_session_util_get_message_bytes (message);
+
+		if (bytes) {
+			GError *local_error = NULL;
+
+			if (!e_soup_session_check_result (session, message, bytes->data, bytes->len, &local_error) && local_error) {
+				g_clear_error (error);
+				g_propagate_error (error, local_error);
+			}
+		}
+
 		return NULL;
-
-	expected_length = soup_message_headers_get_content_length (soup_message_get_response_headers (message));
-	if (expected_length > 0)
-		bytes = g_byte_array_sized_new (expected_length > 1024 * 1024 * 10 ? 1024 * 1024 * 10 : expected_length);
-	else
-		bytes = g_byte_array_new ();
-
-	buffer = g_malloc (BUFFER_SIZE);
-
-	while (success = g_input_stream_read_all (input_stream, buffer, BUFFER_SIZE, &nread, cancellable, error),
-	       success && nread > 0) {
-		g_byte_array_append (bytes, buffer, nread);
 	}
 
-	g_free (buffer);
+	bytes = e_soup_session_read_bytes (message, input_stream, cancellable, error);
+
 	g_object_unref (input_stream);
 
+	success = bytes != NULL;
+
 	if (success)
 		success = e_soup_session_check_result (session, message, bytes->data, bytes->len, error);
 
-	if (!success) {
-		g_byte_array_free (bytes, TRUE);
-		bytes = NULL;
-	}
+	if (!success)
+		g_clear_pointer (&bytes, g_byte_array_unref);
 
 	return bytes;
 }
@@ -1649,11 +1805,18 @@ e_soup_session_util_normalize_uri_path (GUri *uri)
 	const gchar *path;
 	gchar **parts, *tmp;
 	GUri *nuri = NULL;
+	GUriFlags flags;
 	gint ii;
 
 	if (!uri)
 		return NULL;
 
+	flags = g_uri_get_flags (uri);
+
+	if ((flags & G_URI_FLAGS_ENCODED) != 0 ||
+	    (flags & G_URI_FLAGS_ENCODED_PATH) != 0)
+		return NULL;
+
 	path = g_uri_get_path (uri);
 
 	if (!*path || g_strcmp0 (path, "/") == 0)
@@ -1999,3 +2162,23 @@ e_soup_session_util_ref_message_request_body (SoupMessage *message,
 
 	return e_input_stream_wrapper_dup (E_INPUT_STREAM_WRAPPER (md->input_stream));
 }
+
+/**
+ * e_soup_session_util_get_message_bytes:
+ * @message: a #SoupMessage
+ *
+ * Returns bytes read from the message response, when the message send failed.
+ * This can be used to examine detailed error returned by the server in
+ * the response body.
+ *
+ * Returns: (transfer none) (nullable): read message data on failed request, or %NULL, when none had been read
+ *
+ * Since: 3.46
+ **/
+GByteArray *
+e_soup_session_util_get_message_bytes (SoupMessage *message)
+{
+	g_return_val_if_fail (SOUP_IS_MESSAGE (message), NULL);
+
+	return g_object_get_data (G_OBJECT (message), E_SOUP_SESSION_MESSAGE_BYTES_KEY);
+}
